#ifndef __EMITTED_VALIDATE_H
#define __EMITTED_VALIDATE_H

#include <bits/stdc++.h>

typedef void (*feedback_function)(const char*, ...);

const int EXITCODE_AC = 0;
const int EXITCODE_WA = 0;

// declare a constant to be used as user_out.
const std::string FILENAME_USER_OUTPUT = "user_out";
// declare a constant to be used as input.
const std::string FILENAME_INPUT = "input";

#define USAGE "./%s\n"
// we have to define USAGE as dummy because we don't want to modify the lib. too much.

std::ifstream judge_in, judge_ans;
// another ifstream must be used for user_out;
std::ifstream user_out;

char *feedbackdir = NULL;

void vreport_feedback(const char* msg, va_list pvar) {
    assert(stderr);
    vfprintf(stderr, msg, pvar);
}

void report_feedback(const char* msg, ...) {
    va_list pvar;
    va_start(pvar, msg);
    vreport_feedback(msg, pvar);
}

void judge_message(const char* msg, ...) {
    va_list pvar;
    va_start(pvar, msg);
    vreport_feedback(msg, pvar);
}

void judge_error(const char* msg, ...) {
    va_list pvar;
    va_start(pvar, msg);
    vreport_feedback(msg, pvar);
    assert(0);
}

// cleaning up.
void cleanup() {
    fflush(stderr);
    fclose(stderr);
    fflush(stdout);
    fclose(stdout);
    judge_in.close();
    user_out.close();
}

// write another function ``scoring()`` for ac or wa.
void scoring(double score) {
    fprintf(stdout, "%.f", score + 1e-9);
}

void accept() {
    scoring(100.);
    cleanup();
    exit(EXITCODE_AC);
}

void accept_with_score(double scorevalue) {
    scoring(scorevalue);
    cleanup();
    exit(EXITCODE_AC);
}

void wrong_answer(const char* msg, ...) {
    va_list pvar;
    va_start(pvar, msg);
    vreport_feedback(msg, pvar);
    scoring(0.);
    cleanup();
    exit(EXITCODE_WA);
}

void init_io() {
    judge_in.open(FILENAME_INPUT, std::ios_base::in);
    if(judge_in.fail()) {
        judge_error("failed to open input file.\n");
    }
    user_out.open(FILENAME_USER_OUTPUT, std::ios_base::in);
    if(user_out.fail()) {
        judge_error("failed to open user output file.\n");
    }
    judge_ans.open("answer", std::ios_base::in);
    if(judge_ans.fail()) {
        judge_error("failed to open answer file.\n");
    }
}
using namespace std;
void OK(istream &ouf, feedback_function waf) {
    string t;
    if (ouf >> t)
        waf("Sai. Du lieu ra khong dung dinh dang.\n");
    judge_message("OK. Dap an duoc chap nhan!\n");
}
struct pii {
    int u, c;
};
typedef vector<pii> vtp;
typedef vector<vtp> vtv;
vtv dsk;
int n, m, s, t, f[1001];


bool check(int u, int v, int c) {
    if (u < 1 || u > n || v < 1 || v > n || c <= 0)
        return 0;
    f[u] -= c; f[v] += c;
    for (auto &x : dsk[u])
        if (x.u == v) {
            int delta = min(c, x.c);
            c -= delta;
            x.c -= delta;
            if (c == 0)
                break;
        }
    if (c)
        return 0;
    return 1;
}

bool check(istream &inf, istream &ans, istream &ouf, feedback_function waf) {
    int u, v, c, as, os;
    ans >> as;
    if (!(ouf >> os))
        waf("Sai. Du lieu ra khong dung!\n");
    if (as != os)
        waf("Sai. Gia tri cua luong khong dung!\n");

    inf >> n >> m >> s >> t;
    memset(f, 0, sizeof f);
    dsk = vtv(n + 1, vtp(0));
    for (int i = 0; i < m; i++) {
        inf >> u >> v >> c;
        dsk[u].push_back({v, c});
    }
    while (ouf >> u >> v >> c)
        if (!check(u, v, c))
            waf("Sai. Luong tren cung (%d->%d) sai!\n", u, v);

    if (f[s] != -os)
        waf("Sai. Tong luong ra khoi dinh phat khac gia tri luong!\n");

    if (f[s] != -os)
        waf("Sai. Tong luong di vao dinh thu khac gia tri luong!\n");

    for (int i = 1; i <= n; i++) {
        if (i == s || i == t)
            continue;
        if (f[i] != 0)
            waf("Sai. Tong luong vao dinh %d khac tong luong ra!\n", i);
    }
    OK(ouf, waf);
    return 1;
}
#endif

int main(void) {
    init_io();
    check(judge_in, judge_ans, user_out, wrong_answer);
    accept();
}
